//---------------------------------------------------------------------------
// TestDensitySeedSurvival.cpp
//---------------------------------------------------------------------------
#include "TestDensitySeedSurvival.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include "GridBase.h"
#include <fstream>
#include <math.h>

////////////////////////////////////////////////////////////////////////////
//  TestDensitySeedSurvival()
////////////////////////////////////////////////////////////////////////////
void clTestDensitySeedSurvival::TestDensitySeedSurvival() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  TestDensityRun1(p_oSimManager);
  TestDensityRun2(p_oSimManager);
  TestDensityRun3(p_oSimManager);

  delete p_oSimManager;
}

/////////////////////////////////////////////////////////////////////////////
//  TestDensityRun1()
/////////////////////////////////////////////////////////////////////////////
void clTestDensitySeedSurvival::TestDensityRun1(clSimManager *p_oSimManager)
{
  clTreePopulation *p_oPop;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oTree;
  clGridBase *p_oSeeds;
  float fValue, fX, fY, fXMin, fXMax, fYMin, fYMax,
        fXCellLength = 8, fYCellLength = 8;
  int *p_iNumSeedlings = new int[4], i;

  //Feed our file to the sim manager
  p_oSimManager->ReadFile(WriteXMLFile1());

  //Add the seeds to the appropriate grid cells
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 100;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(7, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(7, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(7, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 100;
  p_oSeeds->SetValueOfCell(14, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(14, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(14, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(21, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(21, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(21, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 100;
  p_oSeeds->SetValueOfCell(28, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(28, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(28, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(35, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(35, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(35, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(42, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(42, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(42, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 100;
  p_oSeeds->SetValueOfCell(49, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(49, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(49, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(56, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(56, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(56, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(62, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(62, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(62, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(0, 12, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(0, 12, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(0, 12, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(62, 12, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(62, 12, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(62, 12, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 1;
  p_oSeeds->SetValueOfCell(1, 7, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 1;
  p_oSeeds->SetValueOfCell(1, 7, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 1;
  p_oSeeds->SetValueOfCell(1, 7, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  //Run a timestep
  p_oSimManager->RunSim(1);

  //Get the tree population
  p_oPop = (clTreePopulation*) p_oSimManager->GetPopulationObject("treepopulation");

  //Count the number of seedlings in each grid cell
  p_oAllSeedlings = p_oPop->Find("type=1");

  //Check our values for cell 0, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 0;
  fYMax = fYCellLength;
  fXMin = 0 * fXCellLength;
  fXMax = (0 + 1) * fXCellLength;
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX < fXMax && fY >= fYMin && fY < fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(83 == p_iNumSeedlings[1] || 84 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(139 == p_iNumSeedlings[2] || 140 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(181 == p_iNumSeedlings[3] || 182 == p_iNumSeedlings[3]);

  //Check our values for cell 7, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 7 * fXCellLength;
  fXMax = (7 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(259 == p_iNumSeedlings[1] || 260 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(68 == p_iNumSeedlings[2] || 69 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(113 == p_iNumSeedlings[3] || 114 == p_iNumSeedlings[3]);

  //Check our values for cell 14, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 14 * fXCellLength;
  fXMax = (14 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(83 == p_iNumSeedlings[1] || 84 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(139 == p_iNumSeedlings[2] || 140 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(181 == p_iNumSeedlings[3] || 182 == p_iNumSeedlings[3]);

  //Check our values for cell 21, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 21 * fXCellLength;
  fXMax = (21 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(259 == p_iNumSeedlings[1] || 260 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(68 == p_iNumSeedlings[2] || 69 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(113 == p_iNumSeedlings[3] || 114 == p_iNumSeedlings[3]);

  //Check our values for cell 28, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 28 * fXCellLength;
  fXMax = (28 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(83 == p_iNumSeedlings[1] || 84 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(139 == p_iNumSeedlings[2] || 140 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(181 == p_iNumSeedlings[3] || 182 == p_iNumSeedlings[3]);

  //Check our values for cell 35, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 35 * fXCellLength;
  fXMax = (35 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(170 == p_iNumSeedlings[1] || 171 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(212 == p_iNumSeedlings[2] || 213 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(49 == p_iNumSeedlings[3] || 50 == p_iNumSeedlings[3]);

  //Check our values for cell 42, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 42 * fXCellLength;
  fXMax = (42 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(259 == p_iNumSeedlings[1] || 260 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(68 == p_iNumSeedlings[2] || 69 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(113 == p_iNumSeedlings[3] || 114 == p_iNumSeedlings[3]);

  //Check our values for cell 49, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 49 * fXCellLength;
  fXMax = (49 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(83 == p_iNumSeedlings[1] || 84 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(139 == p_iNumSeedlings[2] || 140 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(181 == p_iNumSeedlings[3] || 182 == p_iNumSeedlings[3]);

  //Check our values for cell 56, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 56 * fXCellLength;
  fXMax = (56 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(170 == p_iNumSeedlings[1] || 171 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(212 == p_iNumSeedlings[2] || 213 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(49 == p_iNumSeedlings[3] || 50 == p_iNumSeedlings[3]);

  //Check our values for cell 62, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 62 * fXCellLength;
  fXMax = (62 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(174 == p_iNumSeedlings[1] || 175 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(217 == p_iNumSeedlings[2] || 218 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(56 == p_iNumSeedlings[3] || 57 == p_iNumSeedlings[3]);

  //Check our values for cell 1, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 1 * fXCellLength;
  fXMax = (1 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[3]);

  //Check our values for cell 0, 12
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 12 * fYCellLength;
  fYMax = (12 + 1) * fYCellLength;
  fXMin = 0;
  fXMax = (0 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(174 == p_iNumSeedlings[1] || 175 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(217 == p_iNumSeedlings[2] || 218 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(56 == p_iNumSeedlings[3] || 57 == p_iNumSeedlings[3]);

  //Check our values for cell 62, 12
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 12 * fYCellLength;
  fYMax = (12 + 1) * fYCellLength;
  fXMin = 62 * fXCellLength;
  fXMax = (62 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(177 == p_iNumSeedlings[1] || 178 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(222 == p_iNumSeedlings[2] || 223 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(63 == p_iNumSeedlings[3] || 64 == p_iNumSeedlings[3]);

  //Check our values for cell 1, 7
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 7 * fYCellLength;
  fYMax = (7 + 1) * fYCellLength;
  fXMin = 1 * fYCellLength;
  fXMax = (1 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(1 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(1 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(1 == p_iNumSeedlings[3]);

  delete[] p_iNumSeedlings;
}

/////////////////////////////////////////////////////////////////////////////
//  TestDensityRun2()
/////////////////////////////////////////////////////////////////////////////
void clTestDensitySeedSurvival::TestDensityRun2(clSimManager *p_oSimManager)
{
  clTreePopulation *p_oPop;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oTree;
  clGridBase *p_oSeeds;
  float fValue, fX, fY, fXMin, fXMax, fYMin, fYMax,
        fXCellLength = 3, fYCellLength = 5;
  int *p_iNumSeedlings = new int[4], i;

  //Feed our file to the sim manager
  p_oSimManager->ReadFile(WriteXMLFile2());

  //Add the seeds to the appropriate grid cells
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 100;
  p_oSeeds->SetValueOfCell(1, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(1, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(1, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(20, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(20, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(20, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 100;
  p_oSeeds->SetValueOfCell(38, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(38, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(38, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(57, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(57, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(57, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 100;
  p_oSeeds->SetValueOfCell(76, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(76, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(76, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(94, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(94, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(94, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(113, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(113, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(113, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 100;
  p_oSeeds->SetValueOfCell(132, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(132, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(132, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(150, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(150, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(150, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(166, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(166, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(166, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(1, 19, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(1, 19, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(1, 19, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 200;
  p_oSeeds->SetValueOfCell(166, 19, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(166, 19, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(166, 19, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 1;
  p_oSeeds->SetValueOfCell(4, 12, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 1;
  p_oSeeds->SetValueOfCell(4, 12, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 1;
  p_oSeeds->SetValueOfCell(4, 12, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  //Run a timestep
  p_oSimManager->RunSim(1);

  //Get the tree population
  p_oPop = (clTreePopulation*) p_oSimManager->GetPopulationObject("treepopulation");

  //Count the number of seedlings in each grid cell
  p_oAllSeedlings = p_oPop->Find("type=1");

  //Check our values for cell 1, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 0;
  fYMax = fYCellLength;
  fXMin = 1 * fXCellLength;
  fXMax = (1 + 1) * fXCellLength;
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(4 == p_iNumSeedlings[2] || 5 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(216 == p_iNumSeedlings[3] || 217 == p_iNumSeedlings[3]);

  //Check our values for cell 20, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 20 * fXCellLength;
  fXMax = (20 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(2 == p_iNumSeedlings[2] || 3 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(138 == p_iNumSeedlings[3] || 139 == p_iNumSeedlings[3]);

  //Check our values for cell 38, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 38 * fXCellLength;
  fXMax = (38 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(4 == p_iNumSeedlings[2] || 5 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(216 == p_iNumSeedlings[3] || 217 == p_iNumSeedlings[3]);

  //Check our values for cell 57, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 57 * fXCellLength;
  fXMax = (57 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(2 == p_iNumSeedlings[2] || 3 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(138 == p_iNumSeedlings[3] || 139 == p_iNumSeedlings[3]);

  //Check our values for cell 76, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 76 * fXCellLength;
  fXMax = (76 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(4 == p_iNumSeedlings[2] || 5 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(216 == p_iNumSeedlings[3] || 217 == p_iNumSeedlings[3]);

  //Check our values for cell 94, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 94 * fXCellLength;
  fXMax = (94 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(5 == p_iNumSeedlings[2] || 6 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(63 == p_iNumSeedlings[3] || 64 == p_iNumSeedlings[3]);

  //Check our values for cell 113, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 113 * fXCellLength;
  fXMax = (113 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(2 == p_iNumSeedlings[2] || 3 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(138 == p_iNumSeedlings[3] || 139 == p_iNumSeedlings[3]);

  //Check our values for cell 132, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 132 * fXCellLength;
  fXMax = (132 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(4 == p_iNumSeedlings[2] || 5 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(216 == p_iNumSeedlings[3] || 217 == p_iNumSeedlings[3]);

  //Check our values for cell 150, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 150 * fXCellLength;
  fXMax = (150 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(5 == p_iNumSeedlings[2] || 6 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(63 == p_iNumSeedlings[3] || 64 == p_iNumSeedlings[3]);

  //Check our values for cell 166, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 166 * fXCellLength;
  fXMax = (166 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(4 == p_iNumSeedlings[2] || 5 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(66 == p_iNumSeedlings[3] || 67 == p_iNumSeedlings[3]);

  //Check our values for cell 1, 19
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 19 * fYCellLength;
  fYMax = (19 + 1) * fYCellLength;
  fXMin = 1 * fXCellLength;
  fXMax = (1 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(5 == p_iNumSeedlings[2] || 6 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(63 == p_iNumSeedlings[3] || 64 == p_iNumSeedlings[3]);

  //Check our values for cell 166, 19
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 19 * fYCellLength;
  fYMax = (19 + 1) * fYCellLength;
  fXMin = 166 * fXCellLength;
  fXMax = (166 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(4 == p_iNumSeedlings[2] || 5 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(66 == p_iNumSeedlings[3] || 67 == p_iNumSeedlings[3]);

  //Check our values for cell 4, 12
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 12 * fYCellLength;
  fYMax = (12 + 1) * fYCellLength;
  fXMin = 4 * fXCellLength;
  fXMax = (4 + 1) * fXCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(1 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(1 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(1 == p_iNumSeedlings[3]);

  //Check our values for cell 0, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 0;
  fXMax = fXCellLength;
  fYMin = 0;
  fYMax = fYCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[3]);

  delete[] p_iNumSeedlings;
}

/////////////////////////////////////////////////////////////////////////////
//  TestDensityRun3()
/////////////////////////////////////////////////////////////////////////////
void clTestDensitySeedSurvival::TestDensityRun3(clSimManager *p_oSimManager)
{
  clTreePopulation *p_oPop;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oTree;
  clGridBase *p_oSeeds;
  float fValue, fX, fY, fXMin, fXMax, fYMin, fYMax,
        fXCellLength = 5, fYCellLength = 5;
  int *p_iNumSeedlings = new int[4], i;

  //Feed our file to the sim manager
  p_oSimManager->ReadFile(WriteXMLFile3());

  //Add the seeds to the appropriate grid cells
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 100;
  p_oSeeds->SetValueOfCell(19, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(19, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(19, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(0, 19, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(0, 19, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(0, 19, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 100;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 300;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  fValue = 300;
  p_oSeeds->SetValueOfCell(19, 19, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 100;
  p_oSeeds->SetValueOfCell(19, 19, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 200;
  p_oSeeds->SetValueOfCell(19, 19, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);

  //Run a timestep
  p_oSimManager->RunSim(1);

  //Get the tree population
  p_oPop = (clTreePopulation*) p_oSimManager->GetPopulationObject("treepopulation");

  //Count the number of seedlings in each grid cell
  p_oAllSeedlings = p_oPop->Find("type=1");

  //Check our values for cell 19, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fYMin = 0;
  fYMax = fYCellLength;
  fXMin = 19 * fXCellLength;
  fXMax = (19 + 1) * fXCellLength;
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(85 == p_iNumSeedlings[1] || 86 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(144 == p_iNumSeedlings[2] || 145 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(205 == p_iNumSeedlings[3] || 206 == p_iNumSeedlings[3]);

  //Check our values for cell 0, 19
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 0;
  fXMax = fXCellLength;
  fYMin = 19 * fYCellLength;
  fYMax = (19 + 1) * fYCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(265 == p_iNumSeedlings[1] || 266 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(70 == p_iNumSeedlings[2] || 71 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(130 == p_iNumSeedlings[3] || 131 == p_iNumSeedlings[3]);

  //Check our values for cell 0, 0
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 0;
  fXMax = fXCellLength;
  fYMin = 0;
  fYMax = fYCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(85 == p_iNumSeedlings[1] || 86 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(144 == p_iNumSeedlings[2] || 145 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(205 == p_iNumSeedlings[3] || 206 == p_iNumSeedlings[3]);

  //Check our values for cell 19, 19
  for (i = 0; i < 4; i++) {
    p_iNumSeedlings[i] = 0;
  }
  fXMin = 19 * fXCellLength;
  fXMax = (19 + 1) * fXCellLength;
  fYMin = 19 * fYCellLength;
  fYMax = (19 + 1) * fYCellLength;
  p_oAllSeedlings->StartOver();
  p_oTree = p_oAllSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    if (fX >= fXMin && fX <= fXMax && fY >= fYMin && fY <= fYMax) {
      p_iNumSeedlings[p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllSeedlings->NextTree();
  }
  CPPUNIT_ASSERT(0 == p_iNumSeedlings[0]);
  CPPUNIT_ASSERT(265 == p_iNumSeedlings[1] || 266 == p_iNumSeedlings[1]);
  CPPUNIT_ASSERT(70 == p_iNumSeedlings[2] || 71 == p_iNumSeedlings[2]);
  CPPUNIT_ASSERT(130 == p_iNumSeedlings[3] || 131 == p_iNumSeedlings[3]);

  delete[] p_iNumSeedlings;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestDensitySeedSurvival::WriteXMLFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>5</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>500.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_4\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">40.0</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.0242</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_4\">0.0242</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_4\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.7059</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_4\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.464</tr_sachVal>"
       << "<tr_sachVal species=\"Species_4\">0.464</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_4\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.02871</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0263</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_4\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_4\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_4\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_4\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_4\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_4\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_4\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>";
  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Density Dependent Seed Survival</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seed\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Establishment</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seed\"/>"
       << "</behavior>"


/*       << "<behavior>"
<< "<behaviorName>output</behaviorName>"
<< "<version>1.0</version>"
<< "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
<< "<applyTo species=\"Species_3\" type=\"Seedling\"/>"
<< "<applyTo species=\"Species_4\" type=\"Seedling\"/>"
<< "</behavior>" */


       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_4\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_4\">0</di_nssolVal>"
       << "<di_nssolVal species=\"Species_2\">0</di_nssolVal>"
       << "<di_nssolVal species=\"Species_3\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_4\">0</di_nsiolVal>"
       << "<di_nsiolVal species=\"Species_2\">0</di_nsiolVal>"
       << "<di_nsiolVal species=\"Species_3\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_densDepSlope>"
       << "<es_ddsVal species=\"Species_2\">0.2</es_ddsVal>"
       << "<es_ddsVal species=\"Species_3\">0.4</es_ddsVal>"
       << "<es_ddsVal species=\"Species_4\">0.8</es_ddsVal>"
       << "</es_densDepSlope>"
       << "<es_densDepSteepness>"
       << "<es_ddstVal species=\"Species_2\">-0.2</es_ddstVal>"
       << "<es_ddstVal species=\"Species_3\">-0.1</es_ddstVal>"
       << "<es_ddstVal species=\"Species_4\">-0.3</es_ddstVal>"
       << "</es_densDepSteepness>"
       << "</establishment>"


/*      << "<output>"
<< "<ou_filename>sdf.gz.tar</ou_filename>"
<< "<ou_treeInfo species=\"Species_2\" type=\"Seedling\">"
<< "<ou_saveFreq>1</ou_saveFreq>"
<< "<ou_float>X</ou_float>"
<< "<ou_float>Y</ou_float>"
<< "<ou_float>Diam10</ou_float>"
<< "</ou_treeInfo>"
<< "<ou_treeInfo species=\"Species_3\" type=\"Seedling\">"
<< "<ou_saveFreq>1</ou_saveFreq>"
<< "<ou_float>X</ou_float>"
<< "<ou_float>Y</ou_float>"
<< "<ou_float>Diam10</ou_float>"
<< "</ou_treeInfo>"
<< "<ou_treeInfo species=\"Species_4\" type=\"Seedling\">"
<< "<ou_saveFreq>1</ou_saveFreq>"
<< "<ou_float>X</ou_float>"
<< "<ou_float>Y</ou_float>"
<< "<ou_float>Diam10</ou_float>"
<< "</ou_treeInfo>"
<< "</output>" */

       << "</paramFile>";

  oOut.close();

  return cFileString;
}


/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestDensitySeedSurvival::WriteXMLFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>5</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>500.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_4\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">40.0</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.0242</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_4\">0.0242</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_4\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.7059</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_4\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.464</tr_sachVal>"
       << "<tr_sachVal species=\"Species_4\">0.464</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_4\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.02871</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0263</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_4\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_4\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_4\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_4\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_4\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_4\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_4\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>";

  oOut << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_1\">1</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_2\">2</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_3\">3</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">0</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_lengthXCells>3</ma_lengthXCells>"
       << "<ma_lengthYCells>5</ma_lengthYCells>"
       << "</grid>";

  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Density Dependent Seed Survival</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seed\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Establishment</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_4\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_4\">0</di_nssolVal>"
       << "<di_nssolVal species=\"Species_2\">0</di_nssolVal>"
       << "<di_nssolVal species=\"Species_3\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_4\">0</di_nsiolVal>"
       << "<di_nsiolVal species=\"Species_2\">0</di_nsiolVal>"
       << "<di_nsiolVal species=\"Species_3\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_densDepSlope>"
       << "<es_ddsVal species=\"Species_2\">42.49</es_ddsVal>"
       << "<es_ddsVal species=\"Species_3\">3</es_ddsVal>"
       << "<es_ddsVal species=\"Species_4\">0.8</es_ddsVal>"
       << "</es_densDepSlope>"
       << "<es_densDepSteepness>"
       << "<es_ddstVal species=\"Species_2\">1.87</es_ddstVal>"
       << "<es_ddstVal species=\"Species_3\">0.1</es_ddstVal>"
       << "<es_ddstVal species=\"Species_4\">-0.3</es_ddstVal>"
       << "</es_densDepSteepness>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestDensitySeedSurvival::WriteXMLFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>5</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_4\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">40.0</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.0242</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_4\">0.0242</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_4\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.7059</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_4\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.464</tr_sachVal>"
       << "<tr_sachVal species=\"Species_4\">0.464</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_4\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.02871</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0263</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_4\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_4\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_4\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_4\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_4\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_4\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_4\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>";

  oOut << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_1\">1</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_2\">2</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_3\">3</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">0</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_lengthXCells>5</ma_lengthXCells>"
       << "<ma_lengthYCells>5</ma_lengthYCells>"
       << "</grid>";

  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Density Dependent Seed Survival</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seed\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Establishment</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_4\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_4\">0</di_nssolVal>"
       << "<di_nssolVal species=\"Species_2\">0</di_nssolVal>"
       << "<di_nssolVal species=\"Species_3\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_4\">0</di_nsiolVal>"
       << "<di_nsiolVal species=\"Species_2\">0</di_nsiolVal>"
       << "<di_nsiolVal species=\"Species_3\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_densDepSlope>"
       << "<es_ddsVal species=\"Species_2\">0.2</es_ddsVal>"
       << "<es_ddsVal species=\"Species_3\">0.4</es_ddsVal>"
       << "<es_ddsVal species=\"Species_4\">0.8</es_ddsVal>"
       << "</es_densDepSlope>"
       << "<es_densDepSteepness>"
       << "<es_ddstVal species=\"Species_2\">-0.2</es_ddstVal>"
       << "<es_ddstVal species=\"Species_3\">-0.1</es_ddstVal>"
       << "<es_ddstVal species=\"Species_4\">-0.3</es_ddstVal>"
       << "</es_densDepSteepness>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
