//---------------------------------------------------------------------------
// TestDensitySeedSurvival
//---------------------------------------------------------------------------
#if !defined(TestDensitySeedSurvival_H)
  #define TestDensitySeedSurvival_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clGridBase;
class clSimManager;

/**
* This class tests the density-dependent seed survival behavior.  It writes out
* parameter files and uses them as input to perform setup; it then sets up the
* desired initial conditions, causes the behavior to run, then tests its
* output.  There is one normal processing run.
*/
class clTestDensitySeedSurvival : public CppUnit::TestFixture
{

  public:

  /**
  * Compiles all tests into one function.
  */
  void TestDensitySeedSurvival();

  private:

  /**
  * Tests normal processing of density-dependent seed survival.
  * WriteXMLFile1() does setup.  After a run occurs, the number of seeds in the
  * seed grid is verified.
  */
  void TestDensityRun1(clSimManager *p_oSimManager);

  /**
  * Tests normal processing of density-dependent seed survival with different
  * grid cell sizes. WriteXMLFile2() does setup.  After a run occurs, the
  * number of seeds in the seed grid is verified.
  */
  void TestDensityRun2(clSimManager *p_oSimManager);

  /**
  * Tests normal processing of density-dependent seed survival with different
  * grid cell sizes. WriteXMLFile3() does setup.  After a run occurs, the
  * number of seeds in the seed grid is verified.
  */
  void TestDensityRun3(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  The behaviors are non-spatial
  * disperse (designed to add no seeds) and density-dependent seed survival.
  * There are 4 species but only the last three are used.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing.  The behaviors are non-spatial
  * disperse (designed to add no seeds) and density-dependent seed survival.
  * There are 4 species but only the last three are used.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file to direct testing.  The behaviors are non-spatial
  * disperse (designed to add no seeds) and density-dependent seed survival.
  * There are 4 species but only the last three are used.
  * @return Filename written.
  */
  char * WriteXMLFile3();
};
//---------------------------------------------------------------------------

#endif // TestDensitySeedSurvival_H
