//---------------------------------------------------------------------------
// TestDoubleMMRelGrowth
//---------------------------------------------------------------------------
#if !defined(TestDoubleMMRelGrowth_H)
  #define TestDoubleMMRelGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clDoubleMMRelGrowth class.  This tests both types of double-
* resource relative growth - diam with auto height and diam only
*/
class clTestDoubleMMRelGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one function.
  */
  void TestDoubleMMRelGrowth();

  protected:

  /**
  * Tests the growth when the timestep length is one year.  WriteXMLFile1()
  * is used to set up the run.  10 trees are created of four species,
  * assigned to both growth types.  GLIs are assigned to them, and then growth
  * is run.  Their diameters, heights, and lgm values are checked to make sure
  * they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is three years.  WriteXMLFile1()
  * is used to set up the run.  10 trees are created of four species,
  * assigned to both growth types.  GLIs are assigned to them, and then growth
  * is run.  Their diameters, heights, and lgm values are checked to make sure
  * they are correct.
  *
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  Timestep length is 1 year.
  * This contains four species.
  * Behaviors:
  * <ul>
  * <li>Diam with auto height growth is applied to Species 2 and 4 saplings and
  * adults.</li>
  * <li>Diam only growth is applied to Species 1 and 3 saplings and
  * adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 3 years.
  * This contains four species.
  * Behaviors:
  * <ul>
  * <li>Diam with auto height growth is applied to Species 2 and 4 saplings and
  * adults.</li>
  * <li>Diam only growth is applied to Species 1 and 3 saplings and
  * adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

};
//---------------------------------------------------------------------------
#endif // TestDoubleMMRelGrowth_H
