//---------------------------------------------------------------------------
// TestMerchValueCalculator
//---------------------------------------------------------------------------
#if !defined(TestMerchValueCalculator_H)
  #define TestMerchValueCalculator_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the class clMerchValueCalculator.
*/
class clTestMerchValueCalculator : public CppUnit::TestFixture {

  public:

  /**
  * Performs all the tests from one function.
  */
  void TestMerchValueCalculator();

  protected:

  /**
  * Tests normal processing.  WriteXMLFile1() does setup.  Then 33 trees of 
  * each of 6 species are created.  A simulation is run for one timestep then 
  * the value checked for all those trees, as well as the species totals.  Then
  * all trees are killed, a second timestep is run, and the species total 
  * values are checked to make sure they equal 0.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  This uses the WriteXMLErrorFileX files to 
  * introduce errors and make sure that they are caught.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file for normal processing.  There are 7 species (of
  * which 6 are used), and the merchantable timber value behavior is the only 
  * one in the run.
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes an error file where a form class value is not an allowed value.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error file where the volume behavior is applied to seedlings.
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * Ends with &lt;/trees&gt.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);
};
//---------------------------------------------------------------------------
#endif // TestMerchValueCalculator_H
