//---------------------------------------------------------------------------
// TestPRStormBiLevelGrowth
//---------------------------------------------------------------------------
#if !defined(TestPRStormBiLevelGrowth_H)
  #define TestPRStormBiLevelGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clPRStormBiLevelGrowth class.
*/
class clTestPRStormBiLevelGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one behavior.
  */
  void TestPRStormBiLevelGrowth();

  protected:

  /**
  * Tests normal processing run 1.  WriteXMLFile1() does setup.  Timestep
  * length is 1 year.  The growth used is diam with auto height.  27 trees are
  * created of 3 species and assigned to a mix of light levels and time-since-
  * last-storm.  Then growth is run.  Tree diameters and heights are checked 
  * to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void NormalProcessingRun1(clSimManager *p_oSimManager);

  /**
  * Tests normal processing run 2.  WriteXMLFile2() does setup.  Timestep
  * length is 3 years.  The growth used is diam with auto height.  27 trees are
  * created of 3 species and assigned to a mix of light levels and time-since-
  * last-storm.  Then growth is run.  Tree diameters and heights are checked 
  * to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void NormalProcessingRun2(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  Inputs error files to make sure the errors are
  * caught.
  */
  void ErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file for normal processing run 1.
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file for normal processing run 2.
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes a parameter file where a value for high-light growth threshold is
  * not between 0 and 100.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();
  
  /**
  * Writes a parameter file where there is no "Storm Light" grid (because the
  * storm light behavior is missing).
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();
  
  /**
  * Writes a parameter file where there is no "Storm" grid (because the
  * storm behavior is missing).
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes common portions of the test parameter files.  This starts with
  * &lt;randomSeed&gt; and ends with &lt;/allometry&gt;.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);

  /**
  * Writes growth parameters for the test parameter files.
  * @param oOut File stream to write to.
  */
  void WriteGrowthParameters(std::fstream &oOut);


};
//---------------------------------------------------------------------------
#endif // TestSimpleLinearGrowth_H
